package view;

import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceContext;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.InvalidDnDOperationException;

import javax.swing.JButton;

public class DragButton extends JButton {
  
  public DragButton (String s) {    
    this.setText(s);
    this.setOpaque(true);    
    this.dragSource = DragSource.getDefaultDragSource();
    this.dgListener = new DGListener();
    this.dsListener = new DSListener();
    
    // component, action, listener
    this.dragSource.createDefaultDragGestureRecognizer(
      this,
      this.dragAction,
      this.dgListener); 
  }
  
  public DragButton (DoubleString ds, int a) {
    if (a != DnDConstants.ACTION_NONE &&
      a != DnDConstants.ACTION_COPY &&
      a != DnDConstants.ACTION_MOVE &&
      a != DnDConstants.ACTION_COPY_OR_MOVE &&      
      a != DnDConstants.ACTION_LINK
    ) throw new IllegalArgumentException("action" + a);
    
    this.dragAction = a;
    this.setText(ds.getText());
    this.setName(ds.getName());
    this.setOpaque(true);    
    this.dragSource = DragSource.getDefaultDragSource();
    this.dgListener = new DGListener();
    this.dsListener = new DSListener();
    
    // component, action, listener
    this.dragSource.createDefaultDragGestureRecognizer(
      this,
      this.dragAction,
      this.dgListener); 
  }

  
  /**
   * DGListener
   * a listener that will start the drag.    
   */
  class DGListener implements DragGestureListener {
    /**
     * Start the drag if the operation is ok.
     * uses java.awt.datatransfer.StringSelection to transfer
     * the button's data
     */
    public void dragGestureRecognized(DragGestureEvent e) {

      // if the action is ok we go ahead
      // otherwise we punt
      if((e.getDragAction() & DragButton.this.dragAction) == 0)
	return;
      
      // get the button's text and name and put it inside a Transferable
      DoubleString ds = new DoubleString(DragButton.this.getText(), DragButton.this.getName());
      Transferable transferable = new StringTransferable( ds );

      // now kick off the drag
      try {
	// initial cursor, transferrable, dsource listener      
	e.startDrag(DragSource.DefaultCopyNoDrop,
	  transferable,
	  DragButton.this.dsListener);
	
      }catch( InvalidDnDOperationException idoe ) {
	System.err.println( idoe );
      }
    }
  }
  
  /**
   * DSListener
   * a listener that will track the state of the DnD operation
   * 
   * @see java.awt.dnd.DragSourceListener
   * @see java.awt.dnd.DragSource
   * @see java.awt.datatransfer.StringSelection      
   */
  class DSListener implements DragSourceListener {
    
    /**
     * @param e the event
     */
    public void dragDropEnd(DragSourceDropEvent e) {
      if( e.getDropSuccess() == false ) {
    	  System.out.println( "not successful");
    	  return;
      }

      /*
       * the dropAction should be what the drop target specified
       * in acceptDrop
       */

      // this is the action selected by the drop target
      if(e.getDropAction() == DnDConstants.ACTION_MOVE)
	DragButton.this.setText("");
    }

    /**
     * @param e the event
     */
    public void dragEnter(DragSourceDragEvent e) {
      DragSourceContext context = e.getDragSourceContext();
      //intersection of the users selected action, and the source and target actions
      int myaction = e.getDropAction();
      if( (myaction & DragButton.this.dragAction) != 0) {	
	context.setCursor(DragSource.DefaultCopyDrop);	  
      } else {
	context.setCursor(DragSource.DefaultCopyNoDrop);	  	
      }
    }
    /**
     * @param e the event
     */
    public void dragOver(DragSourceDragEvent e) {
      DragSourceContext context = e.getDragSourceContext();
      int sa = context.getSourceActions();
      int ua = e.getUserAction();
      int da = e.getDropAction();
      int ta = e.getTargetActions();     
    }
    /**
     * @param e the event
     */
    public void dragExit(DragSourceEvent e) { 
      DragSourceContext context = e.getDragSourceContext();
    }

    /**
     * for example, press shift during drag to change to
     * a link action
     * @param e the event     
     */
    public void dropActionChanged (DragSourceDragEvent e) {
      DragSourceContext context = e.getDragSourceContext();      
      context.setCursor(DragSource.DefaultCopyNoDrop);	  	
    }
  }

  private DragSource dragSource;
  private DragGestureListener dgListener;
  private DragSourceListener dsListener;
  private int dragAction = DnDConstants.ACTION_COPY;
}
