package view;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

     
public class StringTransferable implements Transferable, ClipboardOwner {

  public static final DataFlavor plainTextFlavor = DataFlavor.plainTextFlavor;
  public static final DataFlavor localStringFlavor = DataFlavor.stringFlavor;
  public static DataFlavor doubleStringFlavor = new DataFlavor(DoubleString.class, "Non local muutuja");
  public static DataFlavor localDoubleStringFlavor = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType + "; class=view.DoubleString", "Local muutuja");
  
  public static final DataFlavor[] flavors = {
    StringTransferable.plainTextFlavor,
    StringTransferable.localStringFlavor,
    StringTransferable.doubleStringFlavor
  };
  
  private static final List flavorList = Arrays.asList( flavors );
  private String string;
  private DoubleString ds;

  /**
   * Constructor.
   * simply initializes instance variable
   */
  public StringTransferable(String string) {
    this.string = string;
  }
  
  public StringTransferable(DoubleString ds) {
	    this.ds = ds;
	  }
  
  private void dumpFlavor(DataFlavor flavor) {

  }
     
  public synchronized DataFlavor[] getTransferDataFlavors() {
    return flavors;
  }
     
  public boolean isDataFlavorSupported( DataFlavor flavor ) {
    return ( flavorList.contains( flavor ) );
  }
  
  public synchronized Object getTransferData(DataFlavor flavor)
    throws UnsupportedFlavorException, IOException {
    dumpFlavor(flavor);    
    
    if (flavor.equals(StringTransferable.plainTextFlavor)) {
      return new ByteArrayInputStream(this.string.getBytes("Unicode"));
    } else if (StringTransferable.localStringFlavor.equals(flavor)) {
      return this.string;
    } else if (StringTransferable.doubleStringFlavor.equals(flavor)) {
      return this.ds;
    } else if (StringTransferable.localDoubleStringFlavor.equals(flavor)) {
      return this.ds;
    } else {
      throw new UnsupportedFlavorException (flavor);
    }
  }
  public String toString() {
    return "StringTransferable";
  }
  
  public void lostOwnership(Clipboard clipboard, Transferable contents) {
  }
}