package creator;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.JViewport;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

/**
 * 
 * Paneel lahenduskikude ja vigade tabeli avamiseks
 *
 */

public class CheckSolutionsPanel extends JPanel {

	private JPanel container;
	private JTable mistakesTable;
	private JPanel solutionPanel;
	private JButton addMistakesButton;
	private JButton addSolutionsButton;

	CheckSolutionsPanel() {
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		addButtons();
		container = new JPanel();
		add(container);
	}

	private void addButtons() {
		JPanel buttonPanel = new JPanel();
		buttonPanel.setMaximumSize(new Dimension(600, 150));
		JButton solutionButton = new JButton("Ava lahendusfail");
		JButton mistakesButton = new JButton("Ava vigadefail");
		addSolutionsButton = new JButton("Lisa lahendusfail");
		addMistakesButton = new JButton("Lisa vigadefail");
		mistakesButton.setActionCommand("vead");
		mistakesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		solutionButton.setActionCommand("lahendus");
		solutionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		addMistakesButton.setActionCommand("lisa vead");
		addMistakesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		addSolutionsButton.setActionCommand("lisa lahendus");
		addSolutionsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		buttonPanel.add(solutionButton);
		buttonPanel.add(addSolutionsButton);
		buttonPanel.add(mistakesButton);
		buttonPanel.add(addMistakesButton);
		addSolutionsButton.setEnabled(false);
		addMistakesButton.setEnabled(false);
		add(buttonPanel);
	}

	protected void fileMenuActionPerformed(ActionEvent evt) {
		JFileChooser choice = new JFileChooser();
		choice.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int option = choice.showOpenDialog(this);
		if (option == JFileChooser.APPROVE_OPTION) {
			File file = choice.getSelectedFile();
			if (Utils.getExtension(file) == null || !Utils.getExtension(file).equals("txt")) {
				JOptionPane.showMessageDialog(this, "Tegu ei ole .txt laiendiga failiga!", "lesande avamise viga",
						JOptionPane.ERROR_MESSAGE);
			} else if (evt.getActionCommand().equals("vead") || evt.getActionCommand().equals("lisa vead")) {
				{
					if (evt.getActionCommand().equals("vead")) {
						createMistakeTable(file);
					}
					DefaultTableModel model = (DefaultTableModel) mistakesTable.getModel();
					addRows(file, model);
				}
			} else if (evt.getActionCommand().equals("lahendus")) {
				container.removeAll();
				solutionPanel = new JPanel();
				readSolutionFile(file);
				addMistakesButton.setEnabled(false);
				JScrollPane scrollFrame = new JScrollPane(solutionPanel);
				solutionPanel.setAutoscrolls(true);
				scrollFrame.setPreferredSize(new Dimension(700, 600));
				container.add(new JScrollPane(scrollFrame));
				addSolutionsButton.setEnabled(true);
			} else if (evt.getActionCommand().equals("lisa lahendus")) {
				readSolutionFile(file);
			}
		}
		updateUI();
	}

	private void addRows(File file, DefaultTableModel model) {
		ArrayList<String[]> newRows = new ArrayList<String[]>();
		String record;
		try {
			FileReader fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			record = br.readLine();
			if (!record.startsWith("l")) {
				wrongFile();
			} else {
				while ((record = br.readLine()) != null) {
					String[] split = record.replaceFirst("\\:", "\\   ").split("    ");
					newRows.add(split);
					model.addRow(split);
				}
				addMistakesButton.setEnabled(true);
				addSolutionsButton.setEnabled(false);
			}
			br.close();
		} catch (IOException e) {
			System.out.println("Tuli viga!");
			e.printStackTrace();
		}
	}

	private void wrongFile() {
		JOptionPane.showMessageDialog(this, "Ebasobiv fail. Palun vali vead.txt");
	}

	@SuppressWarnings("unchecked")
	private void readSolutionFile(File file) {
		String record;
		try {
			FileReader fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			record = br.readLine();
			ArrayList<ArrayList<String>> lines = new ArrayList<ArrayList<String>>();
			ArrayList<String> tempList = new ArrayList<String>();
			while ((record = br.readLine()) != null) {
				if (record.startsWith("=UUS")) {
					if (tempList.size() > 1) {
						lines.add((ArrayList<String>) tempList.clone());
					}
					tempList.clear();
				}
				tempList.add(record);
			}
			lines.add((ArrayList<String>) tempList.clone());
			fillSolutionPanel(lines);
			br.close();
		} catch (IOException e) {
			System.out.println("Tuli viga!");
			e.printStackTrace();
		}
	}

	private void fillSolutionPanel(ArrayList<ArrayList<String>> lists) {
		solutionPanel.setLayout(new BoxLayout(solutionPanel, BoxLayout.Y_AXIS));
		for (ArrayList<String> list : lists) {
			addSolution(list);
		}
	}

	private void addSolution(ArrayList<String> list) {
		JPanel newPanel = new JPanel();
		newPanel.setLayout(new BoxLayout(newPanel, BoxLayout.Y_AXIS));
		addLabelsToPanel(newPanel, list);
		JPanel yellowPanel = new JPanel();
		yellowPanel.setBackground(Color.yellow);
		yellowPanel.setPreferredSize(new Dimension(5, 5));
		solutionPanel.add(yellowPanel);
		solutionPanel.add(newPanel);
	}

	private void addLabelsToPanel(JPanel newPanel, ArrayList<String> list) {
		JTextPane textPane = new JTextPane();
		HTMLEditorKit kit = new HTMLEditorKit();
		HTMLDocument doc = new HTMLDocument();
		textPane.setEditorKit(kit);
		textPane.setDocument(doc);
		String user = "";
		String problem = "";
		for (int i = 0; i < list.size(); i++) {
			String string = list.get(i);
			if (string.startsWith("=UUS")) {
				string = string.substring(6);
				user = string.split(" ")[0];
				String date = string.split(" ")[1];
				String time = string.split(" ")[2];
				problem = list.get(i + 1).split("_")[0];
				String output = "<b>Lahenduskik</b><hr />Lahendaja: " + user + "<br />Kuupev: " + date + " Kell: "
						+ time + "<br />lesanne: " + problem + "<hr />" + "Kasutaja alustas lahendamist.";
				writeHTML(kit, doc, output);
				continue;
			}
			if (i == 1){
				//teine rida kujul problem_user
				continue;
			}
			if (i == list.size() - 1) {
				if (string.startsWith(user + ": ")) {
					String output = "<b>Lahendaja " + user + " lahendas edukalt lesande " + problem + "!</b>";
					writeHTML(kit, doc, output);
					continue;
				} else {
					String output = "<b><font color=\"red\">Lahendaja " + user + " ei lahendanud lpuni lesannet " + problem
							+ "!</font></b>";
					writeHTML(kit, doc, output);
					continue;
				}
			}
			String nextString = list.get(i + 1);
			if (nextString.startsWith("=UUS") || nextString.startsWith("Ksimus on veel")
					|| nextString.startsWith("Sa tegid arvutamisel") || nextString.startsWith("See ei ole")
					|| nextString.startsWith("Sa oled unustanud") || nextString.startsWith("Moodustatud tehe ei")
					|| nextString.startsWith("Palun sisesta") || nextString.startsWith("Viga mardamisel")
					|| nextString.startsWith("Sellele ksimusele")) {
				continue;
			}
			if (string.startsWith("Ksimus on veel") || string.startsWith("See ei ole")
					|| string.startsWith("Sellele ksimusele")) {
				String output = "<font color=\"red\">Ksimuse moodustamise viga!</font>";
				if (string.startsWith("See ei ole")) {
					output = output.concat(" Moodustati poolik ksimus:<br />" + list.get(i - 1));
				} else if (string.startsWith("Ksimus on veel")) {
					output = output.concat(" Programmi kasutamisel tehti viga, ksimus ji koostamata.");
				} else if (string.startsWith("Sellele ksimusele ei")) {
					output = output.concat(" Moodustatud ksimusele ei saa he sammuga vastata:<br />"
							+ list.get(i - 1));
				} else if (string.startsWith("Sellele ksimusele on")) {
					output = output.concat(" Moodustatud ksimusele on juba vastus olemas:<br />" + list.get(i - 1));
				}
				writeHTML(kit, doc, output);
				continue;
			}
			if (string.startsWith("Sa tegid arvutamisel") || string.startsWith("Viga mardamisel")
					|| string.startsWith("Palun sisesta vastus")) {
				String output = "<font color=\"red\">Arvutusviga.</font>";
				if (string.startsWith("Sa tegid arvutamisel") || string.startsWith("Viga mardamisel")) {
					String previousLine = list.get(i - 1);
					String newLine = previousLine.substring(0, previousLine.lastIndexOf(" ")).concat(" = ")
							.concat(previousLine.substring(previousLine.lastIndexOf(" ") + 1));
					output = output.concat(" Tehe: " + newLine);
				} else {
					output = output.concat(" Vastus ji sisestamata");
				}
				writeHTML(kit, doc, output);
				continue;
			}
			if (string.startsWith("Sa oled unustanud") || string.startsWith("Avaldisest on puudu")
					|| string.startsWith("Moodustatud tehe")) {
				String output = "<font color=\"red\">Avaldise koostamise viga.</font>";
				if (string.startsWith("Sa oled unustanud")) {
					output = output.concat(" Lahendaja unustas tehtemrgi sisestada");
				} else {
					output = output.concat(" " + string);
				}
				writeHTML(kit, doc, output);
				continue;
			}
			writeHTML(kit, doc, "<b> <font color=\"green\">" + string + "</font> </b>");
		}
		newPanel.add(textPane);
	}

	private void writeHTML(HTMLEditorKit kit, HTMLDocument doc, String string) {
		try {
			kit.insertHTML(doc, doc.getLength(), string, 0, 0, null);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void createMistakeTable(File file) {
		container.removeAll();
		String[] columnNames = { "<html>Lahendaja", "<html>lesanne", "<html>Ksimuse koostamise vigu",
				"<html>Arvutusvigu", "<html>Avaldise koostamise vigu", "<html>Vastuse andmise vigu" };
		DefaultTableModel model = new DefaultTableModel(0, 6);
		model.setColumnIdentifiers(columnNames);
		mistakesTable = new JTable(model);
		mistakesTable.setAutoCreateRowSorter(true);
		JScrollPane scrollPane = new JScrollPane(mistakesTable);
		// tabeli pise suuruse muutmine
		// http://stackoverflow.com/questions/11686535/how-to-change-jtable-header-height
		scrollPane.setColumnHeader(new JViewport() {
			@Override
			public Dimension getPreferredSize() {
				Dimension d = super.getPreferredSize();
				d.height = 70;
				return d;
			}
		});
		scrollPane.setPreferredSize(new Dimension(700, 500));
		mistakesTable.setFillsViewportHeight(true);
		container.add(scrollPane);
	}
}
