package creator;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import src.main.java.uk.ac.ed.ph.jacomax.MaximaTimeoutException;

/**
 * Paneel ksimuste vastavusse seadmiseks avaldistega
 */
public class ExpressionPanel extends JPanel implements ActionListener {

	private JPanel expressionMainPanel;
	private JPanel buttonPanel = new JPanel();
	private JPanel questionView = new JPanel();
	private JPanel containerPanel;
	private JPanel relationsPanel;
	private JFrame checkRelationsFrame = new JFrame();
	private JScrollPane scrollPane;
	private JPanel answerPanel;
	private TabbedQuestionPanel tabbedQuestionPanel;
	private ArrayList<String> seosed = new ArrayList<String>();
	private Hashtable<String, String> seosedKysAval = new Hashtable<String, String>();
	private JPanel questionButtonPanel = new JPanel();
	private ButtonGroup group;
	private JButton checkSolutionButton;

	private JButton checkRelationsButton;

	public ExpressionPanel(TabbedQuestionPanel tqp) {
		this.tabbedQuestionPanel = tqp;
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		addButtonsToPanel();
		add(buttonPanel);
		addAnswerPanel();
		addScrollPane();
		setMinimumSize(new Dimension(0, 400));
	}

	// Eraldi paneel lppvastuse ning sellega vastavuses oleva avaldise kuvamiseks
	private void addAnswerPanel() {
		answerPanel = new JPanel();
		answerPanel.add(new JLabel("Vastus: "));
		answerPanel.setMaximumSize(new Dimension(1600, 150));
		JTextField answerText = new JTextField("", 40);
		answerText.setEditable(false);
		answerText.setToolTipText("Kirjuta siia lppksimuse tekst");
		answerPanel.add(answerText);
		answerPanel.add(new JLabel(" = "));
		JTextField answerExpression = new JTextField("", 15);
		answerExpression.setEditable(false);
		answerPanel.add(answerExpression);
		answerPanel.setMinimumSize(new Dimension(100, 100));
		add(answerPanel);
	}

	private void addButtonsToPanel() {
		JButton generateButton = new JButton("Moodusta ksimused");
		checkRelationsButton = new JButton("Kontrolli seoseid");
		checkRelationsButton
				.setToolTipText("Seoste kontrollimiseks vali moodustatud ksimuste seast lppksimus ja -avaldis!");
		checkSolutionButton = CreateSolvabilityButton();
		checkRelationsButton.setEnabled(false);
		checkSolutionButton.setEnabled(false);
		buttonPanel.setMaximumSize(new Dimension(1600, 100));
		buttonPanel.add(generateButton);
		buttonPanel.add(checkRelationsButton);
		checkRelationsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fillRelationHashtable();
				checkRelationsActionPerformed(evt);
			}
		});
		buttonPanel.add(checkSolutionButton);
		generateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				genQuestionsActionPerformed(evt);
			}
		});
	}

	public static List<Map<String, String>> CreateSeosedTable(ArrayList<String> seosedKoos) {
		List<Map<String, String>> listOfMaps = new ArrayList<Map<String, String>>();
		Hashtable<String, String> seosedKysAval = new Hashtable<String, String>();
		Hashtable<String, String> seosedAvalKys = new Hashtable<String, String>();
		for (String avaldis : seosedKoos) {
			if (!avaldis.startsWith("Vastus:")) {
				String[] seos = avaldis.split("=");
				seosedKysAval.put(seos[0], seos[1].replaceAll(":", "/"));
				seosedAvalKys.put(seos[1].replaceAll(":", "/"), seos[0]);
			}
		}
		listOfMaps.add(seosedKysAval);
		listOfMaps.add(seosedAvalKys);
		return listOfMaps;
	}

	// KATRIN; lahenduvuse kontrolli nupu loomine
	public JButton CreateSolvabilityButton() {
		JButton solvabilityButton = new JButton("Kontrolli lahenduvust");
		solvabilityButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fillRelationHashtable();
				List<Map<String, String>> list = CreateSeosedTable(seosed);
				seosedKysAval = (Hashtable<String, String>) list.get(0);
				if (!(seosedKysAval.isEmpty() || seosedKysAval == null)) {
					try {
						String vastusAvaldis = ((JTextField) answerPanel.getComponent(3)).getText();
						ArrayList<String> solution = SearchVertex.AutoSolution(seosedKysAval, vastusAvaldis);
						if (solution != null) {
							JPanel lahendusPanel = new JPanel();
							lahendusPanel.setLayout(new java.awt.GridLayout(0, 1));
							int number = 1;
							Hashtable<String, String> seosedAvalKys = (Hashtable<String, String>) list.get(1);
							JLabel foundLabel = new JLabel("Lahendus leitud! Lahenduskik:", JLabel.CENTER);
							foundLabel.setForeground(new Color(0.1f, 0.8f, 0.15f));
							lahendusPanel.add(foundLabel);

							for (String samm : SearchVertex.forDisplayingSolution(solution, seosedAvalKys)) {
								JPanel sammPanel = new JPanel();
								sammPanel.add(new JLabel(number + ". " + samm.replaceAll("/", ":")));
								lahendusPanel.add(sammPanel);
								number++;
							}

							JFrame lahendusFrame = new JFrame("Lahenduskik");
							lahendusFrame.setResizable(true);
							lahendusFrame.add(lahendusPanel);
							JScrollPane scrollBar = new JScrollPane(lahendusPanel,
									JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
									JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
							lahendusFrame.add(scrollBar);
							lahendusFrame.pack();
							lahendusFrame.setVisible(true);

						} else {
							JOptionPane.showMessageDialog(null, "lesanne ei ole lahenduv.", "Lahenduvus",
									JOptionPane.INFORMATION_MESSAGE);
						}

					} catch (Exception e1) {
						JOptionPane.showMessageDialog(null,
								"Ei saanud hendust programmiga Maxima. Maxima peab asuma kaustas C:\\Program files",
								"Viga", JOptionPane.INFORMATION_MESSAGE);
						e1.printStackTrace();
					}
				}
			}
		});
		return solvabilityButton;
	}

	private void fillRelationHashtable() {
		Component[] questions = questionView.getComponents();
		seosed.clear();
		for (int i = 1; i < questions.length; i += 2) {
			if (!((JTextField) questions[i]).getText().equals("")) {
				seosed.add(((JLabel) questions[i - 1]).getText() + "=" + ((JTextField) questions[i]).getText());
			}
		}
	}

	protected void checkRelationsActionPerformed(ActionEvent evt) {
		checkRelationsFrame.dispose();
		checkRelationsFrame();
	}

	// Seoste kontrollimise akna loomine
	private void checkRelationsFrame() {
		checkRelationsFrame = new JFrame();
		checkRelationsFrame.setMinimumSize(new Dimension(500, 150));
		relationsPanel = new JPanel();
		containerPanel = new JPanel();
		containerPanel.setLayout(new BoxLayout(containerPanel, BoxLayout.PAGE_AXIS));
		containerPanel.add(relationsPanel);
		fillPanel(questionView);
		containerPanel.add(createButtonsPanel());
		checkRelationsFrame.add(new JScrollPane(containerPanel));
		checkRelationsFrame.pack();
		checkRelationsFrame.setVisible(true);
	}

	private void fillPanel(JPanel questionView2) {
		relationsPanel.removeAll();
		relationsPanel.setLayout(new BoxLayout(relationsPanel, BoxLayout.PAGE_AXIS));
		relationsPanel.add(Box.createRigidArea(new Dimension(20, 20)));
		checkForProblems(questionView2.getComponents());
		relationsPanel.updateUI();
		checkRelationsFrame.pack();
	}

	/** Meetod avaldiste kontrolliks */
	private void checkForProblems(Component[] comps) {
		Boolean problem = false;
		Boolean noLabel = true;
		Boolean noAnswer = true;
		String answer = ((JTextField) answerPanel.getComponent(3)).getText();
		for (int counter = 1; counter < comps.length; counter += 2) {
			String expression = ((JTextField) comps[counter]).getText();
			if (!(expression.equals(""))) {
				noLabel = false;
				JLabel label = new JLabel(((JLabel) comps[counter - 1]).getText() + " <--> " + expression);
				Font labelFont = label.getFont();
				label.setFont(new Font(labelFont.getName(), Font.PLAIN, 16));
				int errorAt = syntaxError(expression);
				if (errorAt != expression.length() + 2) {
					problem = true;
					JLabel viga;
					String htmlText;
					if (errorAt < expression.length()) {
						htmlText = expression.substring(0, errorAt - 1) + "<font color='red'>"
								+ expression.charAt(errorAt - 1) + "</font>"
								+ expression.substring(errorAt, expression.length());
						viga = new JLabel("Vigane avaldis. Viga kohal " + errorAt + ".");
					} else {
						htmlText = expression + "<font color='red'>" + "&larr;";
						viga = new JLabel("Vigane avaldis. Viga avaldise lpus.");
					}
					viga.setForeground(Color.RED);
					label.setText("<html>" + ((JLabel) comps[counter - 1]).getText() + " <--> " + htmlText  + "</html>");
					relationsPanel.add(viga);
				} else if (expression.equals(answer)) {
					noAnswer = false;
					JLabel vastus = new JLabel("Lppvastus:");
					vastus.setForeground(new Color(0.1f, 0.8f, 0.15f));
					relationsPanel.add(vastus);
				}
				relationsPanel.add(label);
				label.setAlignmentX(CENTER_ALIGNMENT);
				relationsPanel.add(Box.createRigidArea(new Dimension(20, 20)));
			}
		}
		if (noLabel) {
			JLabel errorLabel = new JLabel("Moodusta ksimused ja sea nendega vastavusse avaldised!");
			errorLabel.setForeground(Color.ORANGE);
			relationCheckError(errorLabel);
			problem = true;
		} else if (noAnswer) {
			JLabel errorLabel = new JLabel("Vastus puudub vi moodustatud seosed ei hti vastusega!");
			errorLabel.setForeground(Color.RED);
			relationCheckError(errorLabel);
			problem = true;
		}
		if (!problem) {
			JLabel checked = new JLabel("Seosed kontrollitud, vigu ei leitud, kontrolli lahenduvust!");
			checked.setForeground(new Color(0.1f, 0.8f, 0.15f));
			checked.setAlignmentX(CENTER_ALIGNMENT);
			relationsPanel.add(checked, 0);
			checkSolutionButton.setEnabled(true);
		}
	}

	/** Avaldiste sntaksikontroll */
	private int syntaxError(String expression) {
		int characterCount = 1;
		byte bracketCounter = 0;
		char previousChar = 0;
		String allowed = "abcdefghijklmnopwrstuvxy+-*/:()";
		String operations = "+-*/:";
		String variables = "abcdefghijklmnopwrstuvxy";
		byte operationCounter = 0;
		for (char c : expression.toLowerCase().toCharArray()) {
			if (!allowed.contains(String.valueOf(c))) {
				return characterCount;
			}
			if (String.valueOf(c).equals("(")) {
				bracketCounter += 1;
				if (variables.contains(String.valueOf(previousChar)) || previousChar == ')') {
					return characterCount;
				}
			}
			if (String.valueOf(c).equals(")")) {
				bracketCounter -= 1;
				if (bracketCounter < 0 || operations.contains(String.valueOf(previousChar)) || previousChar == '(') {
					return characterCount;
				}
			}
			if (operations.contains(String.valueOf(c))) {
				operationCounter -= 1;
				if (operations.contains(String.valueOf(previousChar)) || previousChar == '(') {
					return characterCount;
				}
			}
			if (variables.contains(String.valueOf(c))) {
				operationCounter += 1;
				if (variables.contains(String.valueOf(previousChar)) || previousChar == ')') {
					return characterCount;
				}
			}
			if (operationCounter > 1 || operationCounter < 0) {
				return characterCount;
			}
			characterCount += 1;
			previousChar = c;
		}
		if (bracketCounter != 0 || operationCounter != 1) {
			return characterCount;
		}
		return characterCount + 1;
	}

	private void relationCheckError(JLabel errorLabel) {
		Font labelFont = errorLabel.getFont();
		errorLabel.setFont(new Font(labelFont.getName(), Font.PLAIN, 16));
		relationsPanel.add(errorLabel);
		errorLabel.setAlignmentX(CENTER_ALIGNMENT);
		checkSolutionButton.setEnabled(false);
	}

	private JPanel createButtonsPanel() {
		JPanel buttonsPanel = new JPanel();
		JButton refreshButton = new JButton("Vrskenda");
		refreshButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fillPanel(questionView);
			}
		});
		buttonsPanel.add(refreshButton);
		return buttonsPanel;
	}

	private void genQuestionsActionPerformed(ActionEvent evt) {
		genQuestions();
	}

	//ksimuste genereerimine ning seoste paneelile lisamine
	public void genQuestions() {
		questionButtonPanel.removeAll();
		questionView.removeAll();
		questionView.setLayout(new GridLayout(0, 2));
		JPanel[] panels = tabbedQuestionPanel.getQuestionPanels();
		List<String> questions = questionArray(panels);
		questions.add("");
		for (String question : questions) {
			if (!question.equals("")) {
				JLabel questionField = new JLabel(question);
				questionField.setSize(new Dimension(100, getWidth()));
				questionView.add(questionField);
				JTextField equationField = new JTextField();
				equationField.setSize(new Dimension(100, getWidth()));
				questionView.add(equationField);
				JRadioButton radioButton = new JRadioButton();
				radioButton.setActionCommand(question);
				radioButton.addActionListener(this);
				questionButtonPanel.add(radioButton);
			}
		}
		regroupButtons();
		scrollPane.setViewportView(expressionMainPanel);
	}

	
	private void regroupButtons() {
		group = new ButtonGroup();
		for (Component c : questionButtonPanel.getComponents()) {
			group.add((JRadioButton) c);
		}
	}

	public void actionPerformed(ActionEvent e) {
		setAnswer(e.getActionCommand(), 1);
		int expressionID = 1;
		for (Component c : questionButtonPanel.getComponents()) {
			if (((JRadioButton) c).equals(e.getSource())) {
				setAnswer(((JTextField) questionView.getComponent(expressionID)).getText(), 3);
				checkRelationsButton.setEnabled(true);
			}
			expressionID += 2;
		}
	}

	/* Genereerib tabelitesse sisestatud sonadest kysimused */
	private List<String> questionArray(JPanel[] panels) {
		List<String> questions = new ArrayList<String>();
		for (JPanel panel : panels) {
			questions.addAll((generateQuestions(panel)));
		}
		return questions;
	}

	private List<String> generateQuestions(JPanel panel) {
		List<String> panelQuestions = new ArrayList<String>();
		Component[] components = panel.getComponents();
		int columns = ((GridLayout) panel.getLayout()).getColumns();
		int rows = panel.getComponentCount() / columns;
		int counter = 1;
		panelQuestions = fill(panelQuestions, components, columns, rows, counter);
		return panelQuestions;
	}

	// TODO: refactoring
	private List<String> fill(List<String> panelQuestions, Component[] components, int columns, int rows, int counter) {
		if (counter == 1) {
			for (int rowCount = 1; rowCount <= rows; rowCount += 1) {
				String value = ((JTextField) components[(rowCount - 1) * columns]).getText();
				if (!value.equals("") && !value.equals("*")) {
					panelQuestions.add(value);
				}
				if (value.equals("*")) {
					panelQuestions.add("");
				}
			}
		} else {
			List<String> newQuestions = new ArrayList<String>();
			boolean hasChanged = false;
			for (String question : panelQuestions) {
				for (int rowCount = 1; rowCount <= rows; rowCount += 1) {
					String value = ((JTextField) components[(rowCount - 1) * columns + counter - 1]).getText();
					if (!value.equals("") && !value.equals("*")) {
						hasChanged = true;
						newQuestions.add(question.concat(" " + value));
					}
					if (value.equals("*")) {
						hasChanged = true;
						newQuestions.add(question.concat(""));
					}
				}
			}
			if (hasChanged) {
				panelQuestions = newQuestions;
			}
		}
		if (counter < columns) {
			return fill(panelQuestions, components, columns, rows, counter += 1);
		}
		return panelQuestions;

	}

	private void addScrollPane() {
		expressionMainPanel = new JPanel();
		expressionMainPanel.setLayout(new BorderLayout());
		expressionMainPanel.add(questionButtonPanel, BorderLayout.WEST);
		questionButtonPanel.setLayout(new GridLayout(0, 1));
		expressionMainPanel.add(questionView, BorderLayout.CENTER);
		add(expressionMainPanel);
		scrollPane = new JScrollPane();
		scrollPane.setPreferredSize(new Dimension(800, 320));
		add(scrollPane);
	}

	public JScrollPane getScrollPane() {
		return scrollPane;
	}

	public JPanel getAnswerPanel() {
		return answerPanel;
	}

	public void setAnswer(String string, int n) {
		this.answerPanel.remove(n);
		int fieldSize = 15;
		if (n == 1) {
			fieldSize = 40;
		}
		JTextField answerExpression = new JTextField(string, fieldSize);
		answerExpression.setEditable(false);
		this.answerPanel.add(answerExpression, n);
		updateUI();
	}

	public void setQuestionView(JPanel questionView) {
		this.questionView = questionView;
	}

	public JPanel getQuestionView() {
		return questionView;
	}

	public JButton getCheckSolutionButton() {
		return checkSolutionButton;
	}

	public JButton getCheckRelationsButton() {
		return checkRelationsButton;
	}

	public Component[] getQuestionButtons() {
		return questionButtonPanel.getComponents();
	}
}
