package creator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;

import src.main.java.uk.ac.ed.ph.jacomax.MaximaTimeoutException;

public class Point {
	public Point parent;
	public String expression;
	public ArrayList<String> knownValues;
	public ArrayList<String> knownValuesStd;
	public int[] numberOfVars;
	public static ArrayList<String> existingExps;
	public static String finalAnswer;
	public static ArrayList<String> operators;
	public static ArrayList<String> variables;
	public static int[] numberOfEachVar;
	public static ArrayList<String> userKnownValuesStd;

	public Point(Point parent, String expression, ArrayList<String> knownValues, ArrayList<String> knownValuesStd,
			int[] numberOfVars) {
		this.parent = parent;
		this.expression = expression;
		this.knownValues = knownValues;
		this.knownValuesStd = knownValuesStd;
		this.numberOfVars = numberOfVars;
	}

	// seosedReversed on kujul (ksimus : avaldis)
	public static ArrayList<String> AutoSolution(Hashtable<String, String> seosedReversed, String lppvastus)
			throws MaximaTimeoutException {

		// avaldised, millele on mratud vastavusse ka ksimus
		ArrayList<String> existingExpsTemp = new ArrayList<String>(seosedReversed.values());
		existingExps = new ArrayList<String>();
		for (String expression : existingExpsTemp) {
			existingExps.add(MaximaControl.Standardise(expression));
		}

		finalAnswer = lppvastus.replaceAll(":", "/");
		variables = findVariables(finalAnswer);
		numberOfEachVar = CountVariables(finalAnswer);
		operators = findOperators(finalAnswer);

		Point startPoint = null;
		ArrayList<String> variablesStd = new ArrayList<String>();

		for (String variable : variables) {
			variablesStd.add(MaximaControl.Standardise(variable));
		}

		// algustipus ei ole htegi muutujat veel kasutatud
		int[] startVarCount = new int[variables.size()];
		for (int i = 0; i < variables.size(); i++) {
			startVarCount[i] = 0;
		}
		startPoint = new Point(null, null, variables, variablesStd, startVarCount);

		ArrayList<Point> currentLevel = CreateNextPoints(startPoint);

		// moodustab tasandeid kuni tasandil ei leidu lppvastust, kui tase on
		// thi, siis ei ole lahenduv
		while (FinalAnswerInLevel(currentLevel) == null) {
			if (!currentLevel.isEmpty()) {
				ArrayList<Point> newLevel = CreateNextLevel(currentLevel);
				currentLevel.clear();
				currentLevel.addAll(newLevel);
			} else {
				return null;
			}
		}
		ArrayList<String> lahendusTee = RoadFromBeginning(FinalAnswerInLevel(currentLevel));
		Collections.reverse(lahendusTee);
		return lahendusTee;
	}

	public static ArrayList<String> forDisplayingSolution(ArrayList<String> lahendustee,
			Hashtable<String, String> seosed) throws Exception {
		ArrayList<String> solution = new ArrayList<String>();
		for (String samm : lahendustee) {
			Enumeration<String> avaldised = seosed.keys();
			while (avaldised.hasMoreElements()) {
				String avaldis = avaldised.nextElement();
				if (MaximaControl.AreTheyEqual(samm, avaldis)) {
					solution.add(seosed.get(avaldis) + " = " + avaldis);
				}
			}
		}
		return solution;
	}

	// mitu korda iga muutuja avaldises esineb
	public static int[] CountVariables(String expression) {
		int[] numberOfEachVar = new int[variables.size()];
		for (int i = 0; i < variables.size(); i++) {
			int occurrences = expression.length() - expression.replace(variables.get(i), "").length();
			numberOfEachVar[i] = occurrences;
		}
		return numberOfEachVar;
	}

	// teekond algustipust lppvastuse tipuni
	public static ArrayList<String> RoadFromBeginning(Point finalPoint) {
		ArrayList<String> roadFromBeginning = new ArrayList<String>();
		roadFromBeginning.add(finalPoint.getExpression());
		Point parent = finalPoint.getParent();
		while (parent.getExpression() != null) {
			roadFromBeginning.add(parent.getExpression());
			parent = parent.getParent();
		}
		return roadFromBeginning;
	}

	// moodustab jrgmise tasandi tippe
	public static ArrayList<Point> CreateNextLevel(ArrayList<Point> currentPoints) throws MaximaTimeoutException {
		ArrayList<Point> nextLevel = new ArrayList<Point>();
		for (Point point : currentPoints) {
			ArrayList<Point> nextPoints = CreateNextPoints(point);
			nextLevel.addAll(nextPoints);
			if (FinalAnswerInLevel(nextPoints) != null) { // kui antud tipu
															// alluvates on
															// lppvastus, pole
															// rohkem tippe vaja
															// moodustada
				return nextLevel;
			}
		}
		return nextLevel;
	}

	// tagastab null, kui tase ei sisalda lppvastust, tipu, milles on
	// lppvastus, kui see leidub
	public static Point FinalAnswerInLevel(ArrayList<Point> currentPoints) {
		for (Point point : currentPoints) {
			try {
				if (MaximaControl.AreTheyEqual(point.getExpression(), finalAnswer)) {
					return point;
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	// moodustab tipu jaoks muutujate esinemiste arvu jrjendi senimaani olevas
	// lahenduses
	public static int[] CreateNumberOfVars(String variableLeft, String variableRight, Point parent) {
		int[] parentNumber = parent.getNumberOfVars();
		int[] currentNumber = parentNumber.clone();
		if (variableLeft.length() == 1) {
			int varIndex = variables.indexOf(variableLeft);
			currentNumber[varIndex] = currentNumber[varIndex] + 1;
		}
		if (variableRight.length() == 1) {
			int varIndex = variables.indexOf(variableRight);
			currentNumber[varIndex] = currentNumber[varIndex] + 1;
		}

		return currentNumber;
	}

	// kas tipus on muutujaid kasutatud rohkem kui lppvastuses
	public static boolean DoesPointExceedVarLimit(int[] numberOfVars) {
		for (int i = 0; i < variables.size(); i++) {
			if (numberOfVars[i] > numberOfEachVar[i]) {
				return true;
			}
		}
		return false;
	}

	// koostab he tipu alluvad
	public static ArrayList<Point> CreateNextPoints(Point current) throws MaximaTimeoutException {
		ArrayList<String> knownValues = current.getKnownValues();
		ArrayList<String> knownValuesStd = current.getKnownValuesStd();
		ArrayList<Point> nextPoints = new ArrayList<Point>();

		for (String operator : operators) {
			// + ja * korral ei ole jrjekord oluline
			if (operator.equals("+") || operator.equals("*")) {
				int index = 0;
				for (String value : knownValues) {
					for (int i = index; i < knownValues.size(); i++) {
						String expression = "(" + value + operator + knownValues.get(i) + ")";
						int[] numberOfVars = CreateNumberOfVars(value, knownValues.get(i), current);
						if (DoesPointExceedVarLimit(numberOfVars) == false) {
							String expressionStd = MaximaControl.Standardise(expression);
							if (existingExps.contains(expressionStd) && !knownValuesStd.contains(expressionStd)) {
								ArrayList<String> newKnownValues = new ArrayList<String>();
								newKnownValues.addAll(knownValues);
								newKnownValues.add(expression);
								ArrayList<String> newKnownValuesStd = new ArrayList<String>();
								newKnownValuesStd.addAll(knownValuesStd);
								newKnownValuesStd.add(expressionStd);
								Point point = new Point(current, expression, newKnownValues, newKnownValuesStd,
										numberOfVars);
								nextPoints.add(point);

							}
						}
					}
					index++;
				}

			} else {
				for (String value : knownValues) {
					// - ja / korral on jrjekord oluline
					for (int i = 0; i < knownValues.size(); i++) {
						String expression = "(" + value + operator + knownValues.get(i) + ")";
						int[] numberOfVars = CreateNumberOfVars(value, knownValues.get(i), current);
						if (DoesPointExceedVarLimit(numberOfVars) == false) {
							String expressionStd = MaximaControl.Standardise(expression);
							if (existingExps.contains(expressionStd) && !knownValuesStd.contains(expressionStd)) {
								ArrayList<String> newKnownValues = new ArrayList<String>();
								newKnownValues.addAll(knownValues);
								newKnownValues.add(expression);
								ArrayList<String> newKnownValuesStd = new ArrayList<String>();
								newKnownValuesStd.addAll(knownValuesStd);
								newKnownValuesStd.add(expressionStd);
								Point point = new Point(current, expression, newKnownValues, newKnownValuesStd,
										numberOfVars);
								nextPoints.add(point);

							}
						}
					}
				}
			}

		}
		return nextPoints;
	}

	public Point getParent() {
		return parent;
	}

	public ArrayList<String> getKnownValues() {
		return knownValues;
	}

	public int[] getNumberOfVars() {
		return numberOfVars;
	}

	public ArrayList<String> getKnownValuesStd() {
		return knownValuesStd;
	}

	@Override
	public String toString() {
		return "Point " + knownValues + Arrays.toString(numberOfVars);
	}

	public String getExpression() {
		return expression;
	}

	// kiki muutujaid ei pruugi olla kasutatud lppvastuse saamiseks, neid pole
	// vaja uurida
	public static ArrayList<String> findVariables(String lppvastus) {
		ArrayList<String> muutujad = new ArrayList<String>();

		for (char tht : lppvastus.toCharArray()) {
			if (!(tht == '(' || tht == ')' || tht == '+' || tht == '*' || tht == '/' || tht == ':' || tht == '-')) {
				muutujad.add(Character.toString(tht));
			}
		}
		// eemalda kordused
		HashSet<String> kordusteta = new HashSet<String>();
		kordusteta.addAll(muutujad);
		muutujad.clear();
		muutujad.addAll(kordusteta);
		Collections.sort(muutujad);

		return muutujad;
	}

	// milliseid tehteid lppvastuse saamiseks on vaja
	public static ArrayList<String> findOperators(String lppvastus) {
		ArrayList<String> tehted = new ArrayList<String>();
		if (lppvastus.contains("+")) {
			tehted.add("+");
		}
		if (lppvastus.contains("-")) {
			tehted.add("-");
		}
		if (lppvastus.contains("*")) {
			tehted.add("*");
		}
		if (lppvastus.contains("/")) {
			tehted.add("/");
		}
		return tehted;
	}
}
