package creator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;

import src.main.java.uk.ac.ed.ph.jacomax.MaximaTimeoutException;

/**
 *  Klass, mida kasutatakse lesande lahenduse automaatseks leidmiseks.
 *  Lahendusalgoritmi selgitus on Joosep Norma bakalaureusets
 * 
 * */

public class SearchVertex {

	private static ArrayList<String> existingExps;
	private static String finalAnswer;
	private static ArrayList<String> variables;
	private static ArrayList<String> operators;
	private static ArrayList<String> knownValues;
	private static ArrayList<String> knownValuesStd;
	static HashMap<String, SearchVertex> pointMap;
	private String expression;
	private SearchVertex parent1;
	private SearchVertex parent2;

	public SearchVertex(String exp, SearchVertex p1, SearchVertex p2) {
		this.expression = exp;
		this.parent1 = p1;
		this.parent2 = p2;
	}

	// seosedReversed on kujul (ksimus : avaldis)
	public static ArrayList<String> AutoSolution(Hashtable<String, String> seosedReversed, String lppvastus)
			throws MaximaTimeoutException {

			// avaldised, millele on mratud vastavusse ka ksimus
			ArrayList<String> existingExpsTemp = new ArrayList<String>(seosedReversed.values());
			existingExps = new ArrayList<String>();
			for (String expression : existingExpsTemp) {
				existingExps.add(MaximaControl.Standardise(expression));
			}

			pointMap = new HashMap<String, SearchVertex>();

			knownValues = new ArrayList<String>();
			knownValuesStd = new ArrayList<String>();

			finalAnswer = lppvastus.replaceAll(":", "/");
			variables = findVariables(finalAnswer);
			operators = findOperators(finalAnswer);

			ArrayList<String> variablesStd = new ArrayList<String>();

			for (String variable : variables) {
				knownValues.add(variable);
				String s = MaximaControl.Standardise(variable);
				variablesStd.add(s);
				knownValuesStd.add(s);
			}

			ArrayList<SearchVertex> currentLevel = new ArrayList<SearchVertex>();
			currentLevel = firstLevel(variables);

			// moodustab tasandeid kuni tasandil ei leidu lppvastust, kui tase
			// on
			// thi, siis ei ole lahenduv
			SearchVertex finalPoint = FinalAnswerInLevel(currentLevel);
			while (finalPoint == null && !currentLevel.isEmpty()) {
				ArrayList<SearchVertex> newLevel = CreateNextLevel(currentLevel);
				currentLevel.clear();
				if (!newLevel.isEmpty()) {
					currentLevel.addAll(newLevel);
					finalPoint = FinalAnswerInLevel(currentLevel);
				}
			}

			ArrayList<String> lahendusTee = RoadFromBeginning(finalPoint);
			return lahendusTee;
	}

	private static ArrayList<SearchVertex> firstLevel(ArrayList<String> variables) throws MaximaTimeoutException {
		ArrayList<SearchVertex> firstPoints = new ArrayList<SearchVertex>();
		for (String operator : operators) {
			for (String s : variables) {
				for (String t : variables) {
					String expression = "(" + s + operator + t + ")";
					String expressionStd = MaximaControl.Standardise(expression);
					if (existingExps.contains(expressionStd) && !knownValuesStd.contains(expressionStd)) {
						knownValuesStd.add(expressionStd);
						knownValues.add(expression);
						SearchVertex p = new SearchVertex(expression, null, null);
						firstPoints.add(p);
						pointMap.put(expression, p);
					}
				}
			}
		}
		for (SearchVertex s : firstPoints){
			System.out.println(s.expression);
		}
		return firstPoints;
	}

	private static ArrayList<SearchVertex> CreateNextLevel(ArrayList<SearchVertex> currentLevel) throws MaximaTimeoutException {
		ArrayList<SearchVertex> nextLevel = new ArrayList<SearchVertex>();
		for (SearchVertex p : currentLevel) {
			ArrayList<SearchVertex> points = nextPoints(p);
			if (points.size() > 0) {
				nextLevel.addAll(points);
			}
		}
		System.out.println("------");
		for (SearchVertex s : nextLevel){
			System.out.println(s.expression);
		}
		return nextLevel;
	}

	private static ArrayList<SearchVertex> nextPoints(SearchVertex point) throws MaximaTimeoutException {
		ArrayList<SearchVertex> newPoints = new ArrayList<SearchVertex>();
		String currentExp = point.getExpression();
		ArrayList<String> newKnownValues = (ArrayList<String>) knownValues.clone();
		for (String operator : operators) {
			for (String s : knownValues) {
				String expression = "(" + currentExp + operator + s + ")";
				String expressionStd = MaximaControl.Standardise(expression);
				if (existingExps.contains(expressionStd) && !knownValuesStd.contains(expressionStd)) {
					knownValuesStd.add(expressionStd);
					newKnownValues.add(expression);
					SearchVertex p = new SearchVertex(expression, point, pointMap.get(s));
					newPoints.add(p);
					pointMap.put(expression, p);
				}
				if (operator.equals(":") || operator.equals("/") || operator.equals("-")) {
					String expression2 = "(" + s + operator + currentExp + ")";
					String expression2Std = MaximaControl.Standardise(expression2);
					if (existingExps.contains(expression2Std) && !knownValuesStd.contains(expression2Std)) {
						knownValuesStd.add(expression2Std);
						newKnownValues.add(expression2);
						SearchVertex p = new SearchVertex(expression2, point, pointMap.get(s));
						newPoints.add(p);
						pointMap.put(expression, p);
					}
				}
			}
		}
		knownValues = newKnownValues;
		return newPoints;
	}

	private static ArrayList<String> RoadFromBeginning(SearchVertex finalPoint) {
		ArrayList<String> road = new ArrayList<String>();
		if (finalPoint == null) {
			return null;
		}
		fillRoad(road, finalPoint);
		return road;
	}

	private static void fillRoad(ArrayList<String> road, SearchVertex finalPoint) {
		road.add(0, finalPoint.getExpression());
		if (finalPoint.getParent1() != null) {
			fillRoad(road, finalPoint.getParent1());
		}
		if (finalPoint.getParent2() != null) {
			fillRoad(road, finalPoint.getParent2());
		}
	}

	private SearchVertex getParent2() {
		// TODO Auto-generated method stub
		return parent2;
	}

	private SearchVertex getParent1() {
		// TODO Auto-generated method stub
		return parent1;
	}

	public static SearchVertex FinalAnswerInLevel(ArrayList<SearchVertex> currentPoints) {
		for (SearchVertex point : currentPoints) {
			try {
				if (MaximaControl.AreTheyEqual(point.getExpression(), finalAnswer)) {
					return point;
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	private String getExpression() {
		return expression;
	}

	// kiki muutujaid ei pruugi olla kasutatud lppvastuse saamiseks, neid pole
	// vaja uurida
	public static ArrayList<String> findVariables(String lppvastus) {
		ArrayList<String> muutujad = new ArrayList<String>();

		for (char tht : lppvastus.toCharArray()) {
			if (!(tht == '(' || tht == ')' || tht == '+' || tht == '*' || tht == '/' || tht == ':' || tht == '-')) {
				muutujad.add(Character.toString(tht));
			}
		}
		// eemalda kordused
		HashSet<String> kordusteta = new HashSet<String>();
		kordusteta.addAll(muutujad);
		muutujad.clear();
		muutujad.addAll(kordusteta);
		Collections.sort(muutujad);

		return muutujad;
	}

	// milliseid tehteid lppvastuse saamiseks on vaja
	public static ArrayList<String> findOperators(String lppvastus) {
		ArrayList<String> tehted = new ArrayList<String>();
		if (lppvastus.contains("+")) {
			tehted.add("+");
		}
		if (lppvastus.contains("-")) {
			tehted.add("-");
		}
		if (lppvastus.contains("*")) {
			tehted.add("*");
		}
		if (lppvastus.contains("/")) {
			tehted.add("/");
		}
		return tehted;
	}

	public static ArrayList<String> forDisplayingSolution(ArrayList<String> lahendustee,
			Hashtable<String, String> seosed) throws Exception {
		ArrayList<String> solution = new ArrayList<String>();
		for (String samm : lahendustee) {
			Enumeration<String> avaldised = seosed.keys();
			while (avaldised.hasMoreElements()) {
				String avaldis = avaldised.nextElement();
				if (MaximaControl.AreTheyEqual(samm, avaldis)) {
					solution.add(seosed.get(avaldis) + " = " + samm);
				}
			}
		}
		return solution;
	}
}
