package creator;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.LayoutManager;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

/**
 * Ksimuste moodustamise paneel, mis sisaldab  nelja ksimusteplokki
 */
public class TabbedQuestionPanel extends JPanel {

	private JTabbedPane tabbedPane;
	private JPanel buttonPanel;
	private JPanel questionPanel1;
	private JPanel questionPanel2;
	private JPanel questionPanel3;
	private JPanel questionPanel4;

	// paneel, kuhu kivad tab-ide sees olevad ksimusteplokid
	TabbedQuestionPanel() {
		setMinimumSize(new Dimension(800, 150));
		setPreferredSize(new Dimension(800, 190));
		setMaximumSize(new Dimension(1600, 200));
		setLayout(new BorderLayout());
		tabbedPane = new JTabbedPane();
		tabbedPane.setTabPlacement(JTabbedPane.TOP);
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(0, 1));
		addButtons();
		addTabs(3, 4);
		add(new JLabel("Sisesta iga alalesande vimalikud ksimused. Thja sne sisestamiseks kirjuta lahtrisse *.",
				JLabel.CENTER), BorderLayout.NORTH);
		add(tabbedPane);
		add(buttonPanel, BorderLayout.LINE_END);
	}

	private void addButtons() { 
		JButton addRow = new JButton("Lisa rida");
		addRow.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				addRowActionPerformed(evt);
			}
		});
		JButton deleteRow = new JButton("Kustuta rida");
		deleteRow.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				deleteRowActionPerformed(evt);
			}
		});
		JButton addColumn = new JButton("Lisa veerg");
		addColumn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				addColumnActionPerformed(evt);
			}
		});
		JButton deleteColumn = new JButton("Kustuta veerg");
		deleteColumn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				deleteColumnPerformed(evt);
			}
		});
		buttonPanel.add(addRow);
		buttonPanel.add(deleteRow);
		buttonPanel.add(addColumn);
		buttonPanel.add(deleteColumn);
	}

	// Plokist veeru eemaldamine
	protected void deleteColumnPerformed(ActionEvent evt) {
		int selectedTab = tabbedPane.getSelectedIndex();
		JPanel currentPanel = (JPanel) tabbedPane.getComponentAt(selectedTab);
		int componentCount = currentPanel.getComponentCount();
		int columns = ((GridLayout) currentPanel.getLayout()).getColumns();
		if (columns > 1) {
			int rows = componentCount / columns;
			for (int row = rows; row > 0; row -= 1) {
				currentPanel.remove((row) * columns - 1);
			}
			currentPanel.setLayout(new GridLayout(0, columns - 1));
			updateUI();
		}
	}

	// Plokile veeru lisamine
	protected void addColumnActionPerformed(ActionEvent evt) {
		int selectedTab = tabbedPane.getSelectedIndex();
		JPanel currentPanel = (JPanel) tabbedPane.getComponentAt(selectedTab);
		int componentCount = currentPanel.getComponentCount();
		int columns = ((GridLayout) currentPanel.getLayout()).getColumns();
		int rows = componentCount / columns;
		//TODO: lahendusprogramm ei oska rohkem kui 6 veeruga lesandeid lahendada.
		//kui see ra parandada, vib jrgneva tingimuslause eemaldada.
		if (columns < 6) {
			for (int row = 1; row <= rows; row += 1) {
				currentPanel.add(new JTextField(), row * (columns + 1) - 1);
			}
			currentPanel.setLayout(new GridLayout(0, columns + 1));
		}
		updateUI();
	}
	
	// Plokist rea eemaldamine
	protected void deleteRowActionPerformed(ActionEvent evt) {
		int selectedTab = tabbedPane.getSelectedIndex();
		JPanel currentPanel = (JPanel) tabbedPane.getComponentAt(selectedTab);
		int componentCount = currentPanel.getComponentCount();
		int columns = ((GridLayout) currentPanel.getLayout()).getColumns();
		int rows = componentCount / columns;
		if (rows > 1) {
			for (int c = columns; c > 0; c -= 1) {
				currentPanel.remove(currentPanel.getComponentCount() - 1);
			}
			currentPanel.setLayout(new GridLayout(0, columns));
			updateUI();
		}
	}

	// Plokile rea lisamine
	protected void addRowActionPerformed(ActionEvent evt) {
		int selectedTab = tabbedPane.getSelectedIndex();
		JPanel currentPanel = (JPanel) tabbedPane.getComponentAt(selectedTab);
		int columns = ((GridLayout) currentPanel.getLayout()).getColumns();
		for (int column = 0; column < columns; column += 1) {
			currentPanel.add(new JTextField());
		}
		currentPanel.setLayout(new GridLayout(0, columns));
		updateUI();

	}

	private void addTabs(int questionRows, int questionColumns) {
		questionPanel1 = makeQuestionPanel(questionRows, questionColumns);
		questionPanel2 = makeQuestionPanel(questionRows, questionColumns);
		questionPanel3 = makeQuestionPanel(questionRows, questionColumns);
		questionPanel4 = makeQuestionPanel(questionRows, questionColumns);
		tabbedPane.addTab("1. ksimusteplokk", questionPanel1);
		tabbedPane.addTab("2. ksimusteplokk", questionPanel2);
		tabbedPane.addTab("3. ksimusteplokk", questionPanel3);
		tabbedPane.addTab("4. ksimusteplokk", questionPanel4);
	}

	// kindla arvu ridade ja tulpadega ksimusteploki loomine
	public JPanel makeQuestionPanel(int panelRows, int panelColumns) {
		JPanel questionPanel = new JPanel();
		questionPanel.setLayout(new GridLayout(0, panelColumns));
		for (int i = 1; i <= panelColumns * panelRows; i++) {
			JTextField newField = new JTextField(9);
			questionPanel.add(newField);
		}
		return questionPanel;
	}

	public JTabbedPane getTabbedPane() {
		return tabbedPane;
	}

	public JPanel[] getQuestionPanels() {
		JPanel[] panels = new JPanel[4];
		for (int i = 0; i < 4; i++) {
			panels[i] = (JPanel) tabbedPane.getComponentAt(i);
		}
		return panels;
	}

	public void setQuestionPanel(int panelNumber, JPanel panel) {
		tabbedPane.setComponentAt(panelNumber, panel);
	}

}
