package creator;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.UIManager;
import javax.swing.text.JTextComponent;

/**
 * Kasutajaliidese raam, mille sisse pannakse paneelid
 * 
 */
public class UserInterfaceCreator extends JFrame {

	private JPanel mainPanel;
	private TextPanel textPanel;
	private ExpressionPanel expressionPanel;
	private TabbedQuestionPanel tabbedQuestionPanel;
	private JMenuBar menuBar;
	private JMenuItem saveMenuItem;
	private JMenuItem openMenuItem;

	/** Luuakse vastavalt kasutaja soovile aken ning menriba */
	public UserInterfaceCreator() {
		setTitle("TEKSTERi petajaliides");
		setMinimumSize(new Dimension(720, 740));
		setPreferredSize(new Dimension(800, 800));
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		addMenuBar();
		addMainPanel();
		Object[] options = { "lesande koostamine", "Lahenduste kontrollimine" };
		int n = JOptionPane.showOptionDialog(this, "Palun vali soovitud toiming:", "Teksteri petajaliides",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
		if (n == 1) {
			checkSolutionPanels();
		} else {
			createProblemPanels();
		}
		pack();
		setVisible(true);
	}

	/** Lahenduskikude kontrollimise paneeli loomine */
	private void checkSolutionPanels() {
		mainPanel.removeAll();
		mainPanel.add(new CheckSolutionsPanel());
		saveMenuItem.setEnabled(false);
		openMenuItem.setEnabled(false);
	}

	/** Menriba loomine, kuularite lisamine */
	private void addMenuBar() {
		menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu();
		saveMenuItem = new JMenuItem();
		openMenuItem = new JMenuItem();
		JMenuItem newMenuItem = new JMenuItem();
		JMenuItem closeMenuItem = new JMenuItem();
		fileMenu.setText("File");
		saveMenuItem.setText("Salvesta lesanne");
		openMenuItem.setText("Ava lesanne");
		closeMenuItem.setText("Sulge");
		newMenuItem.setText("Vaheta tegevust");
		fileMenu.add(openMenuItem);
		fileMenu.add(saveMenuItem);
		fileMenu.add(newMenuItem);
		fileMenu.add(closeMenuItem);
		closeMenuItem.setActionCommand("kinni");
		closeMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		openMenuItem.setActionCommand("ava");
		openMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		saveMenuItem.setActionCommand("salvesta");
		saveMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		newMenuItem.setActionCommand("uus");
		newMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fileMenuActionPerformed(evt);
			}
		});
		menuBar.add(fileMenu);
		setJMenuBar(menuBar);
	}

	/** Menriba nuppude funktsioonid */
	private void fileMenuActionPerformed(ActionEvent evt) {
		if (evt.getActionCommand().equals("salvesta")) {
			salvestaYlFaili();
		}
		if (evt.getActionCommand().equals("uus")) {
			this.dispose();
			new UserInterfaceCreator();
		}
		if (evt.getActionCommand().equals("kinni")) {
			System.exit(0);
		}
		if (evt.getActionCommand().equals("ava")) {
			JFileChooser choice = new JFileChooser();
			choice.setFileSelectionMode(JFileChooser.FILES_ONLY);
			int option = choice.showOpenDialog(this);
			if (option == JFileChooser.APPROVE_OPTION) {
				File file = choice.getSelectedFile();
				if (Utils.getExtension(file) == null || !Utils.getExtension(file).equals("txt")) {
					JOptionPane.showMessageDialog(mainPanel, "Tegu ei ole .txt laiendiga failiga!",
							"lesande avamise viga", JOptionPane.ERROR_MESSAGE);
				} else {
					expressionPanel.getCheckSolutionButton().setEnabled(false);
					expressionPanel.getCheckRelationsButton().setEnabled(false);
					loeYl(file);
				}
			}
		}
	}

	/** Olemasoleva lesandefaili sisselugemine */
	// TODO: vajab refaktoreerimist!
	private void loeYl(File file) {
		String record;
		boolean correctFileType = false;
		try {
			FileReader fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			while (true) {
				record = br.readLine();
				if (record.startsWith("-=TEKSTER2=-")) {
					correctFileType = true;
					record = br.readLine();
				}
				if (!record.startsWith("-=TEKSTER2=-") && correctFileType == false) {
					JOptionPane.showMessageDialog(null, "Tegu ei ole igel kujul failiga", "lesande avamise viga",
							JOptionPane.ERROR_MESSAGE);
					break;
				}

				if (record.startsWith("!SEADED=")) {
					record = br.readLine();
					while (!record.startsWith("!")) {
						textPanel.getRangeVariables().add(record);
						record = br.readLine();
					}
				}
				// lesandeteksti sisselugemine
				if (record.startsWith("!TEKST=")) {
					record = br.readLine();
					String problemText = "";
					textPanel.setVariableCounter(0);
					while (!record.startsWith("!")) {
						int variables = record.split("#").length - 1;
						textPanel.setVariableCounter(textPanel.getVariableCounter() + variables);
						problemText = problemText + record + "\n";
						record = br.readLine();
					}
					textPanel.setText(problemText);
				}
				// Ksimusteplokkide titmine ning lisamine
				while (record.startsWith("!VEERUD")) {
					ArrayList<String> array = new ArrayList<String>();
					String columnText = br.readLine();
					int columns = 0;
					int rows = 1;
					while (!columnText.startsWith("!")) {
						array.add(columnText);
						columns += 1;
						int rowCounter = 1;
						for (int i = 0; i < columnText.length(); i += 1) {
							if (columnText.charAt(i) == '&') {
								rowCounter += 1;
							}
						}
						if (rowCounter > rows) {
							rows = rowCounter;
						}
						columnText = br.readLine();
					}
					JPanel newPanel = tabbedQuestionPanel.makeQuestionPanel(rows, columns);
					fillPanel(newPanel, array);
					if (record.contains("2")) {
						tabbedQuestionPanel.setQuestionPanel(1, newPanel);
					} else if (record.contains("3")) {
						tabbedQuestionPanel.setQuestionPanel(2, newPanel);
					} else if (record.contains("4")) {
						tabbedQuestionPanel.setQuestionPanel(3, newPanel);
					} else {
						tabbedQuestionPanel.setQuestionPanel(0, newPanel);
						tabbedQuestionPanel.setQuestionPanel(1, tabbedQuestionPanel.makeQuestionPanel(3, 4));
						tabbedQuestionPanel.setQuestionPanel(2, tabbedQuestionPanel.makeQuestionPanel(3, 4));
						tabbedQuestionPanel.setQuestionPanel(3, tabbedQuestionPanel.makeQuestionPanel(3, 4));
						tabbedQuestionPanel.updateUI();
					}
					record = columnText;
				}
				// Loeb sisse lesandes mratud ksimuste ja avaldiste
				// vastavused
				if (record.startsWith("!VASTAVUSED=")) {
					ArrayList<String> expressions = new ArrayList<String>();
					expressionPanel.genQuestions();
					record = br.readLine();
					expressions.clear();
					while (!record.startsWith("!")) {
						expressions.add(record);
						record = br.readLine();
					}
					if (record.startsWith("!VASTUS=")) {
						record = br.readLine();
						expressionPanel.setAnswer(record, 3);
					}
					fillExpressions(expressions);
					expressionPanel.setAnswer("", 1);
					for (int i = 1; i < expressionPanel.getQuestionView().getComponentCount(); i += 2) {
						if (((JTextField) expressionPanel.getQuestionView().getComponent(i)).getText().equals(record)) {
							expressionPanel.setAnswer(
									((JLabel) expressionPanel.getQuestionView().getComponent(i - 1)).getText(), 1);
							((JRadioButton) expressionPanel.getQuestionButtons()[i / 2]).doClick();
						}
					}
					expressionPanel.updateUI();
					break;
				}
			}
			br.close();

		} catch (IOException e) {
			System.out.println("Tuli viga!");
			e.printStackTrace();
		}
	}

	/**
	 * Meetod, mis vtab sisseloetud faili seostatud avaldised ning lisab need
	 * moodustatud ksimuste paneelis igete ksimuste jrele
	 */
	private void fillExpressions(ArrayList<String> expressions) {
		Component[] components = expressionPanel.getQuestionView().getComponents();
		ArrayList<String> strings = new ArrayList<String>();
		for (Component c : components) {
			String s = "";
			try {
				if (c instanceof JLabel) {
					s = ((JLabel) c).getText();
				} else if (c instanceof JTextField) {
					s = ((JTextField) c).getText();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			strings.add(s);
		}
		for (String expressionString : expressions) {
			String[] split = expressionString.split("=");
			String value = split[0];
			String expression = split[1];
			int place = strings.lastIndexOf(expression);
			if (expressionPanel.getQuestionView().getComponent(place + 1) instanceof JTextField) {
				((JTextField) expressionPanel.getQuestionView().getComponent(place + 1)).setText(value);
			}
		}
	}

	/** Meetod ksimusteploki snadega titmiseks */
	private void fillPanel(JPanel newPanel, ArrayList<String> array) {
		int columns = array.size();
		int column = 0;
		for (String s : array) {
			int row = 0;
			String[] strings = s.split("&"); // TODO:
			for (String cellString : strings) {
				((JTextField) newPanel.getComponent(row * columns + column)).setText(cellString);
				row += 1;
			}
			column += 1;
		}
	}

	private void addMainPanel() {
		mainPanel = new JPanel();
		mainPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		add(mainPanel);
	}

	/** Meetod lesannete koostamiseks vajalike paneelide loomiseks */
	private void createProblemPanels() {
		mainPanel.removeAll();
		textPanel = new TextPanel();
		mainPanel.add(textPanel);
		tabbedQuestionPanel = new TabbedQuestionPanel();
		mainPanel.add(tabbedQuestionPanel);
		expressionPanel = new ExpressionPanel(tabbedQuestionPanel);
		mainPanel.add(expressionPanel);
		saveMenuItem.setEnabled(true);
		openMenuItem.setEnabled(true);
	}

	/**
	 * Meetod tekstlesande faili salvestamiseks
	 */
	// TODO: refactoring
	private void salvestaYlFaili() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setFileFilter(new JFileFilter());
		fileChooser.setDialogTitle("Vali kuhu fail salvestada");
		String failiAsukoht = fileChooser.getCurrentDirectory().toPath().toString();

		if (failiAsukoht != null) {
			fileChooser.setCurrentDirectory(new File(failiAsukoht));
		}
		int userSelection = fileChooser.showSaveDialog(this);
		// Kui on valitud kuhu salvestada siis
		if (userSelection == JFileChooser.APPROVE_OPTION) {
			// salvestamise kausta meeldejtmine
			failiAsukoht = fileChooser.getCurrentDirectory().toPath().toString();
			File fileToSave = fileChooser.getSelectedFile();
			String path = fileToSave.getAbsolutePath();
			if (Utils.getExtension(fileToSave) == null || !Utils.getExtension(fileToSave).equals("txt")) {
				fileToSave = new File(path + ".txt");
			}
			FileWriter fw;
			if (!(textPanel.getTextArea().equals(""))) {
				try {
					fw = new FileWriter(fileToSave, false);
					BufferedWriter bw = new BufferedWriter(fw);
					// kirjutab faili lesande alguse
					bw.write("-=TEKSTER2=-");
					bw.newLine();
					if (textPanel.getRangeVariables().size() > 0) {
						bw.write("!SEADED=");
						bw.newLine();
						for (int variableNumber = 0; variableNumber < textPanel.getRangeVariables().size(); variableNumber += 1) {
							bw.write(textPanel.getRangeVariables().get(variableNumber));
							bw.newLine();
						}
					}
					// teksti kirjutamine
					bw.write("!TEKST=");
					bw.newLine();
					bw.write(textPanel.getTextArea().getText());
					bw.newLine();
					// kui kasutaja on lisanud elemente esimesele ksimuse
					// moodustamise paneelile
					JPanel[] questions = tabbedQuestionPanel.getQuestionPanels();
					for (int panelNumber = 0; panelNumber < 4; panelNumber += 1) {
						if (panelNumber < 4) {
							if (panelNumber == 0) {
								bw.write("!VEERUD=");
								bw.newLine();
							}
							JPanel qPanel = questions[panelNumber];
							int columns = ((GridLayout) qPanel.getLayout()).getColumns();
							int rows = qPanel.getComponentCount() / columns;
							boolean notWritten = true;
							for (int column = 1; column <= columns; column++) {
								String columnConjunction = "";
								for (int row = 1; row <= rows; row++) {
									String cellText = (String) ((JTextComponent) qPanel.getComponent(columns
											* (row - 1) + column - 1)).getText();
									if (!(cellText.equals(""))) {
										if (columnConjunction.equals("")) {
											columnConjunction = cellText;
										} else {
											columnConjunction = columnConjunction + "&" + cellText;
										}
									}
								}
								if (!columnConjunction.equals("")) {
									if (notWritten) {
										if (panelNumber > 0) {
											bw.write("!VEERUD" + (panelNumber + 1) + "=");
											bw.newLine();
										}
										notWritten = false;
									}
									bw.write(columnConjunction);
									bw.newLine();
								}
							}
						}
					}

					// kui kasutaja on moodustanud ksimustele vastavaid
					bw.write("!VASTAVUSED=");
					bw.newLine();
					Component[] comps = expressionPanel.getQuestionView().getComponents();
					for (int counter = 1; counter < comps.length; counter += 2) {
						if (!((JTextField) comps[counter]).getText().equals("")) {
							bw.write(((JTextField) comps[counter]).getText() + "="
									+ ((JLabel) comps[counter - 1]).getText());
							bw.newLine();
						}
					}
					bw.write("!VASTUS=");
					bw.newLine();
					bw.write(((JTextField) expressionPanel.getAnswerPanel().getComponent(3)).getText());
					// lpetab kirjutamise
					bw.flush();
					bw.close();
					fw.close();
					System.gc();
				} catch (IOException e) {
					// tuli viga
					e.printStackTrace();
				}
			}
			System.out.println("Save as file: " + fileToSave.getAbsolutePath());
		}
	}
}
